/*
 * NewOptimalBSTwithRangeAndStategy.java
 *
 * Created on July 23, 2002, 10:40 AM
 */


/**
 *
 * @author  hansen
 * @version 
 */
import java.io.*;
import java.util.*;
import Keys.*;
import Nodes.*;


public class NewOptimalBSTwithRangeAndStrategy extends Object {
    INode root;  // The root of the tree
    ArrayList key;
    ArrayList prob;
    
    double ASTPrime = 0.0;
    
    /** Creates new BinarySearchTree */
    public NewOptimalBSTwithRangeAndStrategy() {
        root = InfiniteRangeNode.getInstance();
    }
    
    
    /** Build an optimal binary search tree from a file of values
     *  and probabilities, including probabilities of misses */
    public void buildOptimalWithMissesFromFile (String filename) {
        String line = "";
        String probstr = "";
        key = new ArrayList();
        key.add(null);
        prob = new ArrayList();
        prob.add(null);
        
        try {
            
            File inFile = new File (filename);
            FileReader fileReader = new FileReader(inFile);
            BufferedReader bufReader = new BufferedReader(fileReader);
            
            /** Read in a value and a probability and insert them into array lists */
            probstr = bufReader.readLine();
            double probab2 = Double.parseDouble(probstr);
            prob.add(new Double(probab2));
            line = bufReader.readLine();
            while (line != null) {
                int value = Integer.parseInt(line);
                key.add(new Integer(value));
                
                probstr = bufReader.readLine();
                double probab = Double.parseDouble(probstr);
                prob.add(new Double(probab));
                
                probstr = bufReader.readLine();
                probab2 = Double.parseDouble(probstr);
                prob.add(new Double(probab2));
                
                line = bufReader.readLine();
            }
        }
        catch (EOFException e) {
            System.out.println("At End of file.");
        }
        catch (IOException e) {
            System.out.println(e.toString());
        }
        catch (NumberFormatException e){
            System.out.println ("Unable to parse " + line);
        }
        
        /** Use the array lists to create an optimal binary search tree
         *   This code is borrowed from Neapolitan, beginning on page 120 */
        int n = prob.size()-1;
        double A [][] = new double [n+2][n+1];
        double probSum[][] = new double [n+2][n+1];
        int R [][] = new int [n+2][n+1];
        
        
        int i, j, k;  // counters
        double t;
        
        
        for (i=1; i<=n; i++){
            A[i][i] = ((Double)prob.get(i)).doubleValue();
            A[i][i-1] = 0;
            R[i][i] = i;
            R[i][i-1] = 0;
        }
        
        A[n+1][n]=0;
        R[n+1][n]=0;

        for (j=0; j<n+1; j++) {
            probSum[0][j] = 0.0;
            probSum[n+1][j] = 0.0;
        }
        
        for (i=0; i<n+2; i++) {
            probSum[i][0] = 0.0;
            probSum[i][n] = 0.0;
        }

        for (int l=1; l<=n; l++)
            for (int m=1; m<=n; m++) {
                double probS = 0.0;
                for (int p=l; p<=m; p++) 
                    probS += ((Double)prob.get(p)).doubleValue();
                probSum[l][m] = probS;
            }
                
        for (int diag=1; diag<=n; diag++)
            for (i=1; i<=n-diag; i++) {
                j = i+diag;
                double sum = 0.0;
                
                // This should be made much more efficient,
                // but it works.
                // Find the minimum cost for location [i][j]
                for (k=i; k<=j; k++)
                    sum = sum + ((Double)prob.get(k)).doubleValue();
                
                
                A[i][j] = A[i][i-1]+A[i+1][j]+ sum;
                R[i][j] = i;
                for (k=i+1; k<=j; k++){
                    if (A[i][k-1] + A[k+1][j] + sum < A[i][j]) {
                        A[i][j] = A[i][k-1] + A[k+1][j] + sum;
                        R[i][j] = k;
                    }
                }
            }

        root = tree(1, n, A, R, probSum);
        System.out.println ("The Average Search Time = " + A[1][n]);
        System.out.println ("The AST Prime = " + ASTPrime);
    }
    
    /** Actually build the tree from the A and R arrays.
     *  this code seems very UNobject-oriented, as we are
     *  setting left and right references someplace besides
     *  in the the Node */
    INode tree(int i, int j, double A[][], int R[][], double probSum[][]) {
        int k;
        INode p;
        
        k = R[i][j];
        double leftProbSum = 0.0;
        double rightProbSum = 0.0;
        if (k!=0) {
            leftProbSum = probSum[i][k-1];
            rightProbSum = probSum[k+1][j];
        }
               
        if (leftProbSum == 0.0 && rightProbSum == 0.0) // leaf 
            if (k%2 == 0) { // Node with a key
                p = new LeafNode (new IntegerKey((Integer)key.get(k/2)));
                ASTPrime += 0.0;  // for completeness
            }
            else {  // Range node
                p = NullNode.getInstance();
                ASTPrime += 0.0;   // for completeness
            }
        else if (leftProbSum == 0.0)  // no left subtree
            if (k%2 == 0) { // node with a key
                p = new ROnlyNode(new IntegerKey((Integer)key.get(k/2)), 
                                                tree(k+1, j, A, R, probSum));
                ASTPrime += rightProbSum + ((Double)prob.get(k)).doubleValue();
            }
            else {
                p = new ROnlyRangeNode(new IntegerKey((Integer)key.get(k/2)), new IntegerKey((Integer)key.get((k+2)/2)),
                                          tree(k+1, j, A, R, probSum));
                ASTPrime += rightProbSum + ((Double)prob.get(k)).doubleValue();                
            }
        else if (rightProbSum == 0.0) { // no right subtree
            if (k%2 == 0) { // Node with a key
                p = new LOnlyNode (new IntegerKey((Integer)key.get(k/2)), tree(i, k-1, A, R, probSum));
                ASTPrime += leftProbSum + ((Double)prob.get(k)).doubleValue(); 
            }
            else {
                p = new LOnlyRangeNode(new IntegerKey((Integer)key.get(k/2)),
                                          new IntegerKey((Integer)key.get((k+2)/2)),
                                         tree(i, k-1, A, R, probSum));
                ASTPrime += leftProbSum + ((Double)prob.get(k)).doubleValue();  
            }
        }

        else if (k%2 == 0) {
            double keyProb = ((Double)prob.get(k)).doubleValue();
            if (leftProbSum >= rightProbSum && leftProbSum >= keyProb)  {
                p = new LNode (new IntegerKey((Integer)key.get(k/2)));
                ASTPrime += leftProbSum + 2.0*(rightProbSum + keyProb);
                p.setLeft(tree(i, k-1, A, R, probSum));
                p.setRight(tree(k+1, j, A, R, probSum));

            }
            else if (rightProbSum >= leftProbSum && rightProbSum >= keyProb) {
                p = new RNode (new IntegerKey((Integer)key.get(k/2)));
                ASTPrime += rightProbSum + 2.0*(leftProbSum + keyProb);
                p.setLeft(tree(i, k-1, A, R, probSum));
                p.setRight(tree(k+1, j, A, R, probSum));
            }
            else {
                p = new NNode (new IntegerKey((Integer)key.get(k/2)));
                ASTPrime += keyProb + 2.0*(leftProbSum + rightProbSum);
                p.setLeft(tree(i, k-1, A, R, probSum));
                p.setRight(tree(k+1, j, A, R, probSum));
            }
        }
        else {
            double keyProb = ((Double)prob.get(k)).doubleValue();
            if (leftProbSum >= rightProbSum) {
                p = new LRangeNode (new IntegerKey((Integer)key.get(k/2)),
                                    new IntegerKey((Integer)key.get((k+2)/2)));
                ASTPrime += leftProbSum + 2.0*(rightProbSum + keyProb);
                p.setLeft(tree(i, k-1, A, R, probSum));
                p.setRight(tree(k+1, j, A, R, probSum));
            }
            else {
                p = new RRangeNode (new IntegerKey((Integer)key.get(k/2)),
                                    new IntegerKey((Integer)key.get((k+2)/2)));
                ASTPrime += rightProbSum + 2.0*(leftProbSum + keyProb);
                p.setLeft(tree(i, k-1, A, R, probSum));
                p.setRight(tree(k+1, j, A, R, probSum));
            }
        }
        return p;
    }
    
    /** search the tree starting at the root */
    public boolean search (Key search_key) {
        return root.search(search_key);
    }
    
    /** Traverse the tree printing out each element */
    public void inOrder() {
        root.inOrder();
    }
    
    /** Traverse the tree in preOrder printing out as we go */
    public void preOrder() {
        root.preOrder();
    }
    
    
    /** a test main for this class (and its subclasses) */
    public static void main (String args[])
    {
        NewOptimalBSTwithRangeAndStrategy bst = new NewOptimalBSTwithRangeAndStrategy();
        bst.buildOptimalWithMissesFromFile("/home/hansen/Research/optimalBST/data/TestExtension4.txt");
        
        bst.preOrder();
        for (int i=0; i<10; i++)
            System.out.println(i + " " + bst.search(new IntegerKey(i)));
        System.out.println();
    }
}