package Nodes;

/*
 * InfiniteRangeNode.java
 *
 * Infinite Range Nodes contain all keys from negative infinity through infinity
 * Created on July 23, 2001, 1:56 PM
 * @author  Stuart Hansen
 */

import Keys.*;

public class InfiniteRangeNode implements INode {

    static private InfiniteRangeNode irn = new InfiniteRangeNode();
    private INode left;   // the left subtree
    private INode right;  // the right subtree
    
    /** Creates new InfiniteRangeNode */
    private InfiniteRangeNode() {
        super();
        left = NullNode.getInstance();
        right = NullNode.getInstance();        
    }
    
    /** There is exactly one range node that goes from -Infinity to Infinity */
    static public InfiniteRangeNode getInstance() {
        return irn;
    }
    
    /** Translate this node to a string */
    public String toString() {
        return (Float.NEGATIVE_INFINITY + "-" + Float.POSITIVE_INFINITY);
    }
    
    /** Searching an empty tree */
    public boolean search (Key searchKey) {
        return false;
    }
    
    /** Inorder traveral and print */
    public void inOrder() {
        System.out.print (" (");
        System.out.print (this);
        System.out.print(")");
    }
    
    /** Preorder traversal and print */
    public void preOrder() {
        System.out.print (" (");
        System.out.print (this);
        System.out.print(")");
    }
    
    /** insert a new key */
    public INode insert (Key newkey) {
        Node n = new Node(newkey);
        n.left = new NegInfRangeNode(newkey);
        n.right = new PosInfRangeNode(newkey);
        return n;
    }
    
    /** setting the left subtree to the argument */
    public void setLeft (INode left) {
        this.left = left;
    }
    
    /** setting the right subtree to the argument */
    public void setRight (INode right) {
        this.right = right;
    }

}
