package Nodes;

/*
 * LOneNode.java
 * Nodes with a key and only a left subtree.
 * Created on July 22, 2002, 3:31 PM
 * @author  Stuart Hansen
 * @version 
 */

import Keys.*;

public class LOnlyNode implements INode {

    private Key key;    // our key
    private INode left; // the left subtree
    
    /** Creates new LOneNode */
    public LOnlyNode(Key key, INode left) {
        this.key = key;
        this.left = left;
    }

    /** Each node must know how to search the subtree that it roots  */
    public boolean search(Key search_key) {
        if (search_key.equals(key))
            return true;
        else
            return left.search(search_key);
    }
    
    /** Each node knows how to insert into the subtree it roots  */
    public INode insert(Key newkey) {
        if (newkey.lt(key)) {
            left = left.insert(newkey);
            return this;
        }
        else {
            INode p = new LeafNode(newkey);
            INode me = new Node(key, left, p);
            return me;
        }   
    }

    /** setLeft sets the left subtree of this node */
    public void setLeft (INode left) {
        this.left = left;
    }
    
    /** setRight sets the right subtree of this node */
    public void setRight (INode right) {
        System.out.println("It is illegal to setRight on a LOnlyNode");
    }
    /** Each node must know how to do an inorder traveral of the tree it roots */
    public void inOrder() {
        System.out.print (" (");
        left.inOrder();
        System.out.print (this);
        System.out.print(")");
    }
    
    public String toString () {
        return ("LO" + key.toString());
    }
    
    /** Each node must know how to do a preorder traversal of the tree it roots  */
    public void preOrder() {
        System.out.print (" (" + this);
        left.preOrder();
        System.out.print(")");
    } 
}
