package Nodes;

/*
 * LOnlyRangeNode.java
 * This class represents range nodes with only left subtrees.
 *
 * Created on July 22, 2002, 3:35 PM
 * @author  Stuart Hansen 
 */

import Keys.*;

public class LOnlyRangeNode implements INode {

    private Key leftKey;  // the left key value
    private Key rightKey;
    
    private INode left;   // the left subtree
    
    /** Creates new LOnlyRangeNode */
    public LOnlyRangeNode(Key leftKey) {
        super();
        this.leftKey = leftKey;
        this.left = NullNode.getInstance();
    }

        /** Creates new LOnlyRangeNode */
    public LOnlyRangeNode(Key leftKey, Key rightKey) {
        super();
        this.leftKey = leftKey;
        this.rightKey = rightKey;
        this.left = NullNode.getInstance();
    }
    
    /** Creates new LOnlyRangeNode */
    public LOnlyRangeNode(Key leftKey, Key rightKey, INode left) {
        super();
        this.leftKey = leftKey;
        this.rightKey = rightKey;
        this.left = left;
    }
    
    /** Search the subtree rooted at this node for the search_key */
    public boolean search (Key searchKey) {
        if (searchKey.le(leftKey))
            return left.search (searchKey);
        else 
            return false;
    }
    
    /* insert a new key */
    public INode insert (Key newkey) {
        if (newkey.le(leftKey))
            return left.insert (newkey);
        else { 
            Node n = new Node(newkey,
                              new RangeNode(leftKey, newkey, left, NullNode.getInstance()),
                              new PosInfRangeNode(newkey));
            return n;
        }
    }
    
        /** setLeft sets the left subtree of this node */
    public void setLeft (INode left) {
        this.left = left;
    }
    
    /** setting the right subtree is of a PosInfRangeNode is illegal */
    public void setRight (INode right) {
        System.out.println("Cannot set the right subtree of a Left Only Range Node.");
    }

    /** Inorder traversal and print */
    public void inOrder() {
        System.out.print (" (");
        left.inOrder();
        System.out.println (this);
        System.out.print(")");
    }
    
    /* Preorder traversal and print */
    public void preOrder() {
        System.out.print (" (" + this);
        left.preOrder();
        System.out.print(")");
    }
    
    public String toString (){
        return ("LOR" + leftKey.toString() +"-" + rightKey.toString());
    }
}
