package Nodes;
/*
 * NegInfRangeNode.java
 * Negative Infinity Range Nodes represent the range of values from negative
 * infinity upto but not including the rightKey.
 * Created on July 23, 2001, 1:30 PM
 * @author  Stuart Hansen 
 */

import Keys.*;

public class NegInfRangeNode implements INode {

    INode right;   // the right subtree
    
    private Key rightKey;  // the right key value

    /** Creates new NegInfRangeNode */
    public NegInfRangeNode(Key rightKey) {
        super();
        this.rightKey = rightKey;
        right = NullNode.getInstance();
    }

    /** translate the node to a string */
    public String toString() {
        return (Float.NEGATIVE_INFINITY + "-" + rightKey);
    }
    
    /** Search the subtree rooted at this node for the search_key */
    public boolean search (Key searchKey) {
        if (!(searchKey.lt(rightKey)))
            return right.search (searchKey);
        else 
            return false;
    }
    
    /** Inorder traversal with print of the tree rooted at this node */
    public void inOrder()
    {
        System.out.print (" (");
        System.out.println (this);
        right.inOrder();
        System.out.print(")");
    }

    /** Preorder traversal with print of the tree rooted at this node */
    public void preOrder() {
        System.out.print (" (" + this);
        right.preOrder();
        System.out.print(")");
    }  
    
    /** insert a new key */
    public INode insert (Key newkey) {
        if (newkey.ge(rightKey))
            return right.insert (newkey);
        else { 
            Node n = new Node(newkey,
                              new NegInfRangeNode(newkey),
                              new RangeNode(newkey, rightKey, NullNode.getInstance(), right));
            return n;
        }
    }
    
    /** setting the left subtree is illegal in a Negative Infinity Range node */
    public void setLeft (INode left) {
        System.out.println ("Cannot set the left subtree of a NegInfRangeNode");
    }
    
    /** set the right subtree of the node to the argument */
    public void setRight (INode right) {
        this.right = right;
    }

}