package Nodes;
/*
 * Node.java
 * Nodes are the standard BST nodes with keys and left and right subtrees
 * @author  Stuart  Hansen
 * @version May 8, 2001
 */

import Keys.*;

public class Node implements INode{

    Key key;      // the key value
    INode left;   // the left subtree
    INode right;  // the right subtree

    
    /** Nodes only exist if they have values to go in them */
    public Node (Key value) {
        super();
        key = value;
        left = NullNode.getInstance();
        right = NullNode.getInstance();
    }
    
    /** Insert a new node with given subtrees */
    public Node (Key value, INode left, INode right) {
        super ();
        key = value;
        this.left = left;
        this.right = right;
    }
    
    /** Convert a node to a string */
    public String toString () {
        return key.toString();
    }
    
    /** Search the subtree rooted at this node for the search_key */
    public boolean search (Key search_key) {
        if (search_key.lt(key))
            return left.search(search_key);
        else if (search_key.gt(key))
            return right.search (search_key);
        else 
            return true;
    }
    
    /** Insert the newkey into the subtree rooted at this node */
    public INode insert (Key newkey) {
        if (newkey.lt(key))
            left = left.insert(newkey);
        else
            right = right.insert(newkey);
        return this;
    }

    /** Inorder traversal and print of the subtree rooted at this node */
    public void inOrder()
    {
        System.out.print (" (");
        left.inOrder();
        System.out.print (this);
        right.inOrder();
        System.out.print(")");
    }

    /** Preorder traversal and print of the subtree rooted at this node */
    public void preOrder() {
        System.out.print (" (" + this);
        left.preOrder();
        right.preOrder();
        System.out.print(")");
    }
    
    /** setLeft sets the left subtree of this node */
    public void setLeft (INode left) {
        this.left = left;
    }
    
    /** setRight sets the right subtree of this node */
    public void setRight (INode right) {
        this.right = right;
    }
}
