package Nodes;
/*
 * NullNode.java
 * NullNodes represent leaves of the BST.
 * They do not contain keys of subtrees.
 * It is implemented using a singleton design pattern.
 * Created on May 8, 2001, 11:14 AM
 *
 * @author  Stuart Hansen
 * @version May 8, 2001
 */

import java.io.*;
import Keys.*;

public class NullNode implements INode {

    /** There is exactly one null node */
    static NullNode instance = new NullNode();
    
    /** The constructor in a singleton is private */
    private NullNode ()
    {
        //
    }
    
    /** We only get the instance.  The client never news one up */
    public static NullNode getInstance() {
        return instance;
    }
    
    /** inserting into a null node creates a new non-null node */
    public INode insert (Key newkey) {
        return new Node(newkey);
    }
    
    public String toString() {
        return "nil";
    }
    
    /** There ain't nothin to traverse */
    public void inOrder () {
        System.out.print (" nil");
        return;
    }
    
    /** There ain't nothin to traverse */
    public void preOrder () {
        System.out.print(" nil");
        return;
    }
    
    /** Searching a null node returns false */ 
    public boolean search (Key searchkey) {
        return false;
    }
        
    /** setLeft is illegal in NullNodes */
    public void setLeft (INode left) {
        System.out.println ("NullNodes cannot contain subtrees.");
    }
    
    /** setRight is illegal in NullNodes */
    public void setRight(INode right) {
        System.out.println ("NullNodes cannot contain subtrees.");
    }
}
