package Nodes;
/*
 * PosInfRangeNode.java
 * Represent ranges from the leftKey through positive Infinity
 *
 * Created on July 23, 2001, 1:41 PM
 * @author  Stuart Hansen
 */

import Keys.*;

public class PosInfRangeNode implements INode {

    
    private Key leftKey;  // the left key value    
    private INode left;   // the left subtree
    
    /** Creates new PosInfRangeNode */
    public PosInfRangeNode(Key leftKey) {
        super();
        this.leftKey = leftKey;
        left = NullNode.getInstance();
    }

    /** Translate this node into a string */
    public String toString() {
        return (leftKey + "-" + Float.POSITIVE_INFINITY);
    }
    
    /** Search the subtree rooted at this node for the search_key */
    public boolean search (Key searchKey) {
        if (!(searchKey.gt(leftKey)))
            return left.search (searchKey);
        else 
            return false;
    }
    
    /* insert a new key */
    public INode insert (Key newkey) {
        if (newkey.le(leftKey))
            return left.insert (newkey);
        else { 
            Node n = new Node(newkey,
                              new RangeNode(leftKey, newkey, left, NullNode.getInstance()),
                              new PosInfRangeNode(newkey));
            return n;
        }
    }
    
        /** setLeft sets the left subtree of this node */
    public void setLeft (INode left) {
        this.left = left;
    }
    
    /** setting the right subtree is of a PosInfRangeNode is illegal */
    public void setRight (INode right) {
        System.out.println("Cannot set the right subtree of a Positive Infinity Range Node.");
    }

    /** Inorder traversal and print */
    public void inOrder() {
        System.out.print (" (");
        left.inOrder();
        System.out.println (this);
        System.out.print(")");
    }
    
    /* Preorder traversal and print */
    public void preOrder() {
        System.out.print (" (" + this);
        left.preOrder();
        System.out.print(")");
    }
}