package Nodes;

/*
 * ROnlyRangeNode.java
 * This class is for range nodes with only right subtrees
 *
 * Created on July 22, 2002, 3:36 PM
 * @author  Stuart Hansen
 * @version 
 */

import Keys.*;

public class ROnlyRangeNode implements INode {

    INode right;   // the right subtree 
    private Key leftKey;
    private Key rightKey;  // the right key value

    /** Creates new ROnlyRangeNode */
    public ROnlyRangeNode(Key rightKey) {
        super();
        this.rightKey = rightKey;
        this.right = NullNode.getInstance();
    }

    /** Creates new ROnlyRangeNode */
    public ROnlyRangeNode(Key leftKey, Key rightKey) {
        super();
        this.leftKey = leftKey;
        this.rightKey = rightKey;
        this.right = NullNode.getInstance();
    }
    
    /** Creates new ROnlyRangeNode */
    public ROnlyRangeNode(Key leftKey, Key rightKey, INode right) {
        super();
        this.leftKey = leftKey;
        this.rightKey = rightKey;
        this.right = right;
    }

    /** translate the node to a string */
    public String toString() {
        return ("ROR" + leftKey.toString() + "-" + rightKey.toString());
    }
    
    /** Search the subtree rooted at this node for the search_key */
    public boolean search (Key searchKey) {
        if (!(searchKey.lt(rightKey)))
            return right.search (searchKey);
        else 
            return false;
    }
    
    /** Inorder traversal with print of the tree rooted at this node */
    public void inOrder()
    {
        System.out.print (" (");
        System.out.println (this);
        right.inOrder();
        System.out.print(")");
    }

    /** Preorder traversal with print of the tree rooted at this node */
    public void preOrder() {
        System.out.print (" (" + this);
        right.preOrder();
        System.out.print(")");
    }  
    
    /** insert a new key */
    public INode insert (Key newkey) {
        if (newkey.ge(rightKey))
            return right.insert (newkey);
        else { 
            Node n = new Node(newkey,
                              new ROnlyRangeNode(leftKey, newkey),  // is this correct???
                              new RangeNode(newkey, rightKey, NullNode.getInstance(), right));
            return n;
        }
    }
    
    /** setting the left subtree is illegal in a Negative Infinity Range node */
    public void setLeft (INode left) {
        System.out.println ("Cannot set the left subtree of a ROnlyRangeNode");
    }
    
    /** set the right subtree of the node to the argument */
    public void setRight (INode right) {
        this.right = right;
    }
}
