package Nodes;

/*
 * RangeNode.java
 * RangeNodes represent ranges of keys that are not present in the tree.
 *
 * @author  Stuart  Hansen
 * @version May 8, 2001
 */

import Keys.*;

public class RangeNode implements INode {

    Key left_key;   // the left key value
    Key right_key;  // the right key value
    
    INode left;     // the left subtree
    INode right;    // the right subtree
    
    
    
    /** Nodes only exist if they have values to go in them */
    public RangeNode(Key left_key, Key right_key) {
        super();
        this.left_key = left_key;
        this.right_key = right_key;
        left = NullNode.getInstance();
        right = NullNode.getInstance();
    }
    
    /** This constructor assumes the left and right subtrees already exist */
    public RangeNode (Key left_key, Key right_key, INode left, INode right) {
        super();
        this.left_key = left_key;
        this.right_key = right_key;
        this.left = left;
        this.right = right;
    }
    
    /** translate this node into a string */
    public String toString() {
        return (left_key + "-" + right_key);
    }
    
    /** Search the subtree rooted at this node for the search_key */
    public boolean search (Key search_key) {
        if (!(search_key.gt(left_key)))
            return left.search(search_key);
        else if (!(search_key.lt(right_key)))
            return right.search (search_key);
        else 
            return false;
    }
    
    /** Inorder traversal and print */
    public void inOrder() {
        System.out.print (" (");
        left.inOrder();
        System.out.println (this);
        right.inOrder();
        System.out.print(")");
    }
    
    /* Preorder traversal and print */
    public void preOrder() {
        System.out.print (" (" + this);
        left.preOrder();
        right.preOrder();
        System.out.print(")");
    }
    
    /** Insert a new key */
    public INode insert (Key newkey) {
        if (newkey.le(left_key))
            return left.insert(newkey);
        else if (newkey.ge(right_key))
            return right.insert (newkey);
        else { 
            Node n = new Node(newkey,
                              new RangeNode(left_key, newkey, left, NullNode.getInstance()),
                              new RangeNode(newkey, right_key, NullNode.getInstance(), right) );
            return n;
        }
    }       
    
    /** setLeft sets the left subtree of this node */
    public void setLeft (INode left) {
        this.left = left;
    }
    
    /** setRight sets the right subtree of this node */
    public void setRight (INode right) {
        this.right = right;
    }

}
